#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class AccessType:
    INVALID: Enum
    BYTE: Enum
    SHORT: Enum
    WORD: Enum
    DOUBLEWORD: Enum
    QUADWORD: Enum
    FLOAT: Enum
    DOUBLEFLOAT: Enum
    WORD_LEFT: Enum
    WORD_RIGHT: Enum
    DOUBLEWORD_LEFT: Enum
    DOUBLEWORD_RIGHT: Enum
    MAX: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

__version_info__: tuple[int, int, int]
__version__: str
__author__: str

from .Utils import *

from .Enum import *
from .InstrCategory import *
from .InstrId import *
from .InstrIdType import *
from .OperandType import *
from .AccessType import *

from .RegGprO32 import *
from .RegGprN32 import *

from .RegCop1O32 import *
from .RegCop1N32 import *
from .RegCop1N64 import *

from .Abi import *

from .TrinaryValue import *

from .Config import *

from .rabbitizer import *

from .JrRegData import *
from .RegistersTracker import *



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class Utils:
    @staticmethod
    def from2Complement(number: int, bits: int) -> int: ...

    @staticmethod
    def escapeString(src: str) -> str: ...

    @staticmethod
    def floatRepr_32From16(hex_repr: int) -> int:
        """
        Converts an unsigned 16 bits value representing a half precision
        float to a unsigned 32 bits value representing a single precision
        float.
        """



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class RegistersTracker:
    registerNum: int

    hasLuiValue: bool
    luiOffset: int # The offset of last lui which set a value to this register
    luiSetOnBranchLikely: bool

    hasGpGot: bool
    gpGotOffset: int

    hasLoValue: bool
    loOffset: int
    dereferenced: bool
    dereferenceOffset: int

    value: int



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegGprO32:
    zero: Enum
    at: Enum
    v0: Enum
    v1: Enum
    a0: Enum
    a1: Enum
    a2: Enum
    a3: Enum
    t0: Enum
    t1: Enum
    t2: Enum
    t3: Enum
    t4: Enum
    t5: Enum
    t6: Enum
    t7: Enum
    s0: Enum
    s1: Enum
    s2: Enum
    s3: Enum
    s4: Enum
    s5: Enum
    s6: Enum
    s7: Enum
    t8: Enum
    t9: Enum
    k0: Enum
    k1: Enum
    gp: Enum
    sp: Enum
    fp: Enum
    ra: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class Abi:
    NUMERIC: Enum
    O32: Enum
    N32: Enum
    N64: Enum
    @staticmethod
    def fromStr(name: str | None) -> Enum: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class InstrId:
    cpu_INVALID: Enum
    cpu_j: Enum
    cpu_jal: Enum
    cpu_beq: Enum
    cpu_bne: Enum
    cpu_beql: Enum
    cpu_bnel: Enum
    cpu_blez: Enum
    cpu_blezl: Enum
    cpu_bgtz: Enum
    cpu_bgtzl: Enum
    cpu_addi: Enum
    cpu_addiu: Enum
    cpu_slti: Enum
    cpu_sltiu: Enum
    cpu_andi: Enum
    cpu_ori: Enum
    cpu_xori: Enum
    cpu_daddi: Enum
    cpu_daddiu: Enum
    cpu_lui: Enum
    cpu_ldl: Enum
    cpu_ldr: Enum
    cpu_lb: Enum
    cpu_lh: Enum
    cpu_lwl: Enum
    cpu_lw: Enum
    cpu_lbu: Enum
    cpu_lhu: Enum
    cpu_lwr: Enum
    cpu_lwu: Enum
    cpu_sb: Enum
    cpu_sh: Enum
    cpu_swl: Enum
    cpu_sw: Enum
    cpu_sdl: Enum
    cpu_sdr: Enum
    cpu_swr: Enum
    cpu_ll: Enum
    cpu_pref: Enum
    cpu_lld: Enum
    cpu_ld: Enum
    cpu_sc: Enum
    cpu_scd: Enum
    cpu_sd: Enum
    cpu_cache: Enum
    cpu_lwc1: Enum
    cpu_ldc1: Enum
    cpu_swc1: Enum
    cpu_sdc1: Enum
    cpu_lwc2: Enum
    cpu_ldc2: Enum
    cpu_swc2: Enum
    cpu_sdc2: Enum
    cpu_b: Enum
    cpu_beqz: Enum
    cpu_bnez: Enum
    cpu_sll: Enum
    cpu_srl: Enum
    cpu_sra: Enum
    cpu_dsll: Enum
    cpu_dsrl: Enum
    cpu_dsra: Enum
    cpu_dsll32: Enum
    cpu_dsrl32: Enum
    cpu_dsra32: Enum
    cpu_dsllv: Enum
    cpu_dsrlv: Enum
    cpu_dsrav: Enum
    cpu_sllv: Enum
    cpu_srlv: Enum
    cpu_srav: Enum
    cpu_mthi: Enum
    cpu_mtlo: Enum
    cpu_jr: Enum
    cpu_jalr: Enum
    cpu_mfhi: Enum
    cpu_mflo: Enum
    cpu_movz: Enum
    cpu_movn: Enum
    cpu_div: Enum
    cpu_divu: Enum
    cpu_sn64_div: Enum
    cpu_sn64_divu: Enum
    cpu_ddiv: Enum
    cpu_ddivu: Enum
    cpu_add: Enum
    cpu_addu: Enum
    cpu_sub: Enum
    cpu_subu: Enum
    cpu_and: Enum
    cpu_or: Enum
    cpu_xor: Enum
    cpu_nor: Enum
    cpu_slt: Enum
    cpu_sltu: Enum
    cpu_dadd: Enum
    cpu_daddu: Enum
    cpu_dsub: Enum
    cpu_dsubu: Enum
    cpu_syscall: Enum
    cpu_break: Enum
    cpu_sync: Enum
    cpu_mult: Enum
    cpu_multu: Enum
    cpu_dmult: Enum
    cpu_dmultu: Enum
    cpu_tge: Enum
    cpu_tgeu: Enum
    cpu_tlt: Enum
    cpu_tltu: Enum
    cpu_teq: Enum
    cpu_tne: Enum
    cpu_nop: Enum
    cpu_move: Enum
    cpu_not: Enum
    cpu_neg: Enum
    cpu_negu: Enum
    cpu_bltz: Enum
    cpu_bgez: Enum
    cpu_bltzl: Enum
    cpu_bgezl: Enum
    cpu_tgei: Enum
    cpu_tgeiu: Enum
    cpu_tlti: Enum
    cpu_tltiu: Enum
    cpu_teqi: Enum
    cpu_tnei: Enum
    cpu_bltzal: Enum
    cpu_bgezal: Enum
    cpu_bltzall: Enum
    cpu_bgezall: Enum
    cpu_bal: Enum
    cpu_mfc0: Enum
    cpu_dmfc0: Enum
    cpu_cfc0: Enum
    cpu_mtc0: Enum
    cpu_dmtc0: Enum
    cpu_ctc0: Enum
    cpu_bc0f: Enum
    cpu_bc0t: Enum
    cpu_bc0fl: Enum
    cpu_bc0tl: Enum
    cpu_tlbr: Enum
    cpu_tlbwi: Enum
    cpu_tlbwr: Enum
    cpu_tlbp: Enum
    cpu_rfe: Enum
    cpu_eret: Enum
    cpu_mfc1: Enum
    cpu_dmfc1: Enum
    cpu_mtc1: Enum
    cpu_dmtc1: Enum
    cpu_cfc1: Enum
    cpu_ctc1: Enum
    cpu_bc1f: Enum
    cpu_bc1t: Enum
    cpu_bc1fl: Enum
    cpu_bc1tl: Enum
    cpu_add_s: Enum
    cpu_sub_s: Enum
    cpu_mul_s: Enum
    cpu_div_s: Enum
    cpu_sqrt_s: Enum
    cpu_abs_s: Enum
    cpu_mov_s: Enum
    cpu_neg_s: Enum
    cpu_round_l_s: Enum
    cpu_trunc_l_s: Enum
    cpu_ceil_l_s: Enum
    cpu_floor_l_s: Enum
    cpu_round_w_s: Enum
    cpu_trunc_w_s: Enum
    cpu_ceil_w_s: Enum
    cpu_floor_w_s: Enum
    cpu_cvt_d_s: Enum
    cpu_cvt_w_s: Enum
    cpu_cvt_l_s: Enum
    cpu_c_f_s: Enum
    cpu_c_un_s: Enum
    cpu_c_eq_s: Enum
    cpu_c_ueq_s: Enum
    cpu_c_olt_s: Enum
    cpu_c_ult_s: Enum
    cpu_c_ole_s: Enum
    cpu_c_ule_s: Enum
    cpu_c_sf_s: Enum
    cpu_c_ngle_s: Enum
    cpu_c_seq_s: Enum
    cpu_c_ngl_s: Enum
    cpu_c_lt_s: Enum
    cpu_c_nge_s: Enum
    cpu_c_le_s: Enum
    cpu_c_ngt_s: Enum
    cpu_add_d: Enum
    cpu_sub_d: Enum
    cpu_mul_d: Enum
    cpu_div_d: Enum
    cpu_sqrt_d: Enum
    cpu_abs_d: Enum
    cpu_mov_d: Enum
    cpu_neg_d: Enum
    cpu_round_l_d: Enum
    cpu_trunc_l_d: Enum
    cpu_ceil_l_d: Enum
    cpu_floor_l_d: Enum
    cpu_round_w_d: Enum
    cpu_trunc_w_d: Enum
    cpu_ceil_w_d: Enum
    cpu_floor_w_d: Enum
    cpu_cvt_s_d: Enum
    cpu_cvt_w_d: Enum
    cpu_cvt_l_d: Enum
    cpu_c_f_d: Enum
    cpu_c_un_d: Enum
    cpu_c_eq_d: Enum
    cpu_c_ueq_d: Enum
    cpu_c_olt_d: Enum
    cpu_c_ult_d: Enum
    cpu_c_ole_d: Enum
    cpu_c_ule_d: Enum
    cpu_c_df_d: Enum
    cpu_c_ngle_d: Enum
    cpu_c_seq_d: Enum
    cpu_c_ngl_d: Enum
    cpu_c_lt_d: Enum
    cpu_c_nge_d: Enum
    cpu_c_le_d: Enum
    cpu_c_ngt_d: Enum
    cpu_cvt_s_w: Enum
    cpu_cvt_d_w: Enum
    cpu_cvt_s_l: Enum
    cpu_cvt_d_l: Enum
    cpu_mfc2: Enum
    cpu_mtc2: Enum
    cpu_cfc2: Enum
    cpu_ctc2: Enum
    cpu_MAX: Enum
    rsp_INVALID: Enum
    rsp_mfc2: Enum
    rsp_mtc2: Enum
    rsp_cfc2: Enum
    rsp_ctc2: Enum
    rsp_vmulf: Enum
    rsp_vmulu: Enum
    rsp_vrndp: Enum
    rsp_vmulq: Enum
    rsp_vmudl: Enum
    rsp_vmudm: Enum
    rsp_vmudn: Enum
    rsp_vmudh: Enum
    rsp_vmacf: Enum
    rsp_vmacu: Enum
    rsp_vrndn: Enum
    rsp_vmacq: Enum
    rsp_vmadl: Enum
    rsp_vmadm: Enum
    rsp_vmadn: Enum
    rsp_vmadh: Enum
    rsp_vadd: Enum
    rsp_vsub: Enum
    rsp_vabs: Enum
    rsp_vaddc: Enum
    rsp_vsubc: Enum
    rsp_vsar: Enum
    rsp_vand: Enum
    rsp_vnand: Enum
    rsp_vor: Enum
    rsp_vnor: Enum
    rsp_vxor: Enum
    rsp_vnxor: Enum
    rsp_vlt: Enum
    rsp_veq: Enum
    rsp_vne: Enum
    rsp_vge: Enum
    rsp_vcl: Enum
    rsp_vch: Enum
    rsp_vcr: Enum
    rsp_vmrg: Enum
    rsp_vrcp: Enum
    rsp_vrcpl: Enum
    rsp_vrcph: Enum
    rsp_vmov: Enum
    rsp_vrsq: Enum
    rsp_vrsql: Enum
    rsp_vrsqh: Enum
    rsp_vnop: Enum
    rsp_lbv: Enum
    rsp_lsv: Enum
    rsp_llv: Enum
    rsp_ldv: Enum
    rsp_lqv: Enum
    rsp_lrv: Enum
    rsp_lpv: Enum
    rsp_luv: Enum
    rsp_lhv: Enum
    rsp_lfv: Enum
    rsp_ltv: Enum
    rsp_sbv: Enum
    rsp_ssv: Enum
    rsp_slv: Enum
    rsp_sdv: Enum
    rsp_sqv: Enum
    rsp_srv: Enum
    rsp_spv: Enum
    rsp_suv: Enum
    rsp_shv: Enum
    rsp_sfv: Enum
    rsp_stv: Enum
    rsp_swv: Enum
    rsp_j: Enum
    rsp_jal: Enum
    rsp_beq: Enum
    rsp_bne: Enum
    rsp_blez: Enum
    rsp_bgtz: Enum
    rsp_addi: Enum
    rsp_addiu: Enum
    rsp_slti: Enum
    rsp_sltiu: Enum
    rsp_andi: Enum
    rsp_ori: Enum
    rsp_xori: Enum
    rsp_lui: Enum
    rsp_lb: Enum
    rsp_lh: Enum
    rsp_lw: Enum
    rsp_lbu: Enum
    rsp_lhu: Enum
    rsp_sb: Enum
    rsp_sh: Enum
    rsp_sw: Enum
    rsp_pref: Enum
    rsp_b: Enum
    rsp_beqz: Enum
    rsp_bnez: Enum
    rsp_sll: Enum
    rsp_srl: Enum
    rsp_sra: Enum
    rsp_sllv: Enum
    rsp_srlv: Enum
    rsp_srav: Enum
    rsp_jr: Enum
    rsp_jalr: Enum
    rsp_movz: Enum
    rsp_movn: Enum
    rsp_add: Enum
    rsp_addu: Enum
    rsp_sub: Enum
    rsp_subu: Enum
    rsp_and: Enum
    rsp_or: Enum
    rsp_xor: Enum
    rsp_nor: Enum
    rsp_slt: Enum
    rsp_sltu: Enum
    rsp_break: Enum
    rsp_nop: Enum
    rsp_move: Enum
    rsp_not: Enum
    rsp_neg: Enum
    rsp_negu: Enum
    rsp_bltz: Enum
    rsp_bgez: Enum
    rsp_bltzal: Enum
    rsp_bgezal: Enum
    rsp_bal: Enum
    rsp_mfc0: Enum
    rsp_mtc0: Enum
    rsp_MAX: Enum
    r3000gte_INVALID: Enum
    r3000gte_rtps: Enum
    r3000gte_rtpt: Enum
    r3000gte_dpcl: Enum
    r3000gte_dpcs: Enum
    r3000gte_dpct: Enum
    r3000gte_intpl: Enum
    r3000gte_ncs: Enum
    r3000gte_nct: Enum
    r3000gte_ncds: Enum
    r3000gte_ncdt: Enum
    r3000gte_nccs: Enum
    r3000gte_ncct: Enum
    r3000gte_cdp: Enum
    r3000gte_cc: Enum
    r3000gte_nclip: Enum
    r3000gte_avsz3: Enum
    r3000gte_avsz4: Enum
    r3000gte_mvmva: Enum
    r3000gte_sqr: Enum
    r3000gte_op: Enum
    r3000gte_gpf: Enum
    r3000gte_gpl: Enum
    r3000gte_MAX: Enum
    r4000allegrex_INVALID: Enum
    r4000allegrex_lv_s: Enum
    r4000allegrex_sv_s: Enum
    r4000allegrex_lv_q: Enum
    r4000allegrex_sv_q: Enum
    r4000allegrex_clz: Enum
    r4000allegrex_clo: Enum
    r4000allegrex_madd: Enum
    r4000allegrex_maddu: Enum
    r4000allegrex_msub: Enum
    r4000allegrex_msubu: Enum
    r4000allegrex_max: Enum
    r4000allegrex_min: Enum
    r4000allegrex_srl: Enum
    r4000allegrex_rotr: Enum
    r4000allegrex_srlv: Enum
    r4000allegrex_rotrv: Enum
    r4000allegrex_sleep: Enum
    r4000allegrex_mfie: Enum
    r4000allegrex_mtie: Enum
    r4000allegrex_ext: Enum
    r4000allegrex_ins: Enum
    r4000allegrex_wsbh: Enum
    r4000allegrex_wsbw: Enum
    r4000allegrex_seb: Enum
    r4000allegrex_seh: Enum
    r4000allegrex_bitrev: Enum
    r4000allegrex_bvf: Enum
    r4000allegrex_bvt: Enum
    r4000allegrex_bvfl: Enum
    r4000allegrex_bvtl: Enum
    r4000allegrex_mfv: Enum
    r4000allegrex_mfvc: Enum
    r4000allegrex_vsync2: Enum
    r4000allegrex_mtv: Enum
    r4000allegrex_mtvc: Enum
    r4000allegrex_vadd_s: Enum
    r4000allegrex_vadd_p: Enum
    r4000allegrex_vadd_t: Enum
    r4000allegrex_vadd_q: Enum
    r4000allegrex_vsub_s: Enum
    r4000allegrex_vsub_p: Enum
    r4000allegrex_vsub_t: Enum
    r4000allegrex_vsub_q: Enum
    r4000allegrex_vsbn_s: Enum
    r4000allegrex_vdiv_s: Enum
    r4000allegrex_vdiv_p: Enum
    r4000allegrex_vdiv_t: Enum
    r4000allegrex_vdiv_q: Enum
    r4000allegrex_vmul_s: Enum
    r4000allegrex_vmul_p: Enum
    r4000allegrex_vmul_t: Enum
    r4000allegrex_vmul_q: Enum
    r4000allegrex_vdot_p: Enum
    r4000allegrex_vdot_t: Enum
    r4000allegrex_vdot_q: Enum
    r4000allegrex_vscl_p: Enum
    r4000allegrex_vscl_t: Enum
    r4000allegrex_vscl_q: Enum
    r4000allegrex_vhdp_p: Enum
    r4000allegrex_vhdp_t: Enum
    r4000allegrex_vhdp_q: Enum
    r4000allegrex_vcrs_t: Enum
    r4000allegrex_vdet_p: Enum
    r4000allegrex_vcmp_s: Enum
    r4000allegrex_vcmp_p: Enum
    r4000allegrex_vcmp_t: Enum
    r4000allegrex_vcmp_q: Enum
    r4000allegrex_vmin_s: Enum
    r4000allegrex_vmin_p: Enum
    r4000allegrex_vmin_t: Enum
    r4000allegrex_vmin_q: Enum
    r4000allegrex_vmax_s: Enum
    r4000allegrex_vmax_p: Enum
    r4000allegrex_vmax_t: Enum
    r4000allegrex_vmax_q: Enum
    r4000allegrex_vscmp_s: Enum
    r4000allegrex_vscmp_p: Enum
    r4000allegrex_vscmp_t: Enum
    r4000allegrex_vscmp_q: Enum
    r4000allegrex_vsge_s: Enum
    r4000allegrex_vsge_p: Enum
    r4000allegrex_vsge_t: Enum
    r4000allegrex_vsge_q: Enum
    r4000allegrex_vslt_s: Enum
    r4000allegrex_vslt_p: Enum
    r4000allegrex_vslt_t: Enum
    r4000allegrex_vslt_q: Enum
    r4000allegrex_vwbn_s: Enum
    r4000allegrex_vmov_s: Enum
    r4000allegrex_vmov_p: Enum
    r4000allegrex_vmov_t: Enum
    r4000allegrex_vmov_q: Enum
    r4000allegrex_vabs_s: Enum
    r4000allegrex_vabs_p: Enum
    r4000allegrex_vabs_t: Enum
    r4000allegrex_vabs_q: Enum
    r4000allegrex_vneg_s: Enum
    r4000allegrex_vneg_p: Enum
    r4000allegrex_vneg_t: Enum
    r4000allegrex_vneg_q: Enum
    r4000allegrex_vidt_p: Enum
    r4000allegrex_vidt_q: Enum
    r4000allegrex_vsat0_s: Enum
    r4000allegrex_vsat0_p: Enum
    r4000allegrex_vsat0_t: Enum
    r4000allegrex_vsat0_q: Enum
    r4000allegrex_vsat1_s: Enum
    r4000allegrex_vsat1_p: Enum
    r4000allegrex_vsat1_t: Enum
    r4000allegrex_vsat1_q: Enum
    r4000allegrex_vzero_s: Enum
    r4000allegrex_vzero_p: Enum
    r4000allegrex_vzero_t: Enum
    r4000allegrex_vzero_q: Enum
    r4000allegrex_vone_s: Enum
    r4000allegrex_vone_p: Enum
    r4000allegrex_vone_t: Enum
    r4000allegrex_vone_q: Enum
    r4000allegrex_vrcp_s: Enum
    r4000allegrex_vrcp_p: Enum
    r4000allegrex_vrcp_t: Enum
    r4000allegrex_vrcp_q: Enum
    r4000allegrex_vrsq_s: Enum
    r4000allegrex_vrsq_p: Enum
    r4000allegrex_vrsq_t: Enum
    r4000allegrex_vrsq_q: Enum
    r4000allegrex_vsin_s: Enum
    r4000allegrex_vsin_p: Enum
    r4000allegrex_vsin_t: Enum
    r4000allegrex_vsin_q: Enum
    r4000allegrex_vcos_s: Enum
    r4000allegrex_vcos_p: Enum
    r4000allegrex_vcos_t: Enum
    r4000allegrex_vcos_q: Enum
    r4000allegrex_vexp2_s: Enum
    r4000allegrex_vexp2_p: Enum
    r4000allegrex_vexp2_t: Enum
    r4000allegrex_vexp2_q: Enum
    r4000allegrex_vlog2_s: Enum
    r4000allegrex_vlog2_p: Enum
    r4000allegrex_vlog2_t: Enum
    r4000allegrex_vlog2_q: Enum
    r4000allegrex_vsqrt_s: Enum
    r4000allegrex_vsqrt_p: Enum
    r4000allegrex_vsqrt_t: Enum
    r4000allegrex_vsqrt_q: Enum
    r4000allegrex_vasin_s: Enum
    r4000allegrex_vasin_p: Enum
    r4000allegrex_vasin_t: Enum
    r4000allegrex_vasin_q: Enum
    r4000allegrex_vnrcp_s: Enum
    r4000allegrex_vnrcp_p: Enum
    r4000allegrex_vnrcp_t: Enum
    r4000allegrex_vnrcp_q: Enum
    r4000allegrex_vnsin_s: Enum
    r4000allegrex_vnsin_p: Enum
    r4000allegrex_vnsin_t: Enum
    r4000allegrex_vnsin_q: Enum
    r4000allegrex_vrexp2_s: Enum
    r4000allegrex_vrexp2_p: Enum
    r4000allegrex_vrexp2_t: Enum
    r4000allegrex_vrexp2_q: Enum
    r4000allegrex_vrnds_s: Enum
    r4000allegrex_vrndi_s: Enum
    r4000allegrex_vrndi_p: Enum
    r4000allegrex_vrndi_t: Enum
    r4000allegrex_vrndi_q: Enum
    r4000allegrex_vrndf1_s: Enum
    r4000allegrex_vrndf1_p: Enum
    r4000allegrex_vrndf1_t: Enum
    r4000allegrex_vrndf1_q: Enum
    r4000allegrex_vrndf2_s: Enum
    r4000allegrex_vrndf2_p: Enum
    r4000allegrex_vrndf2_t: Enum
    r4000allegrex_vrndf2_q: Enum
    r4000allegrex_vf2h_p: Enum
    r4000allegrex_vf2h_q: Enum
    r4000allegrex_vh2f_s: Enum
    r4000allegrex_vh2f_p: Enum
    r4000allegrex_vsbz_s: Enum
    r4000allegrex_vlgb_s: Enum
    r4000allegrex_vuc2ifs_s: Enum
    r4000allegrex_vc2i_s: Enum
    r4000allegrex_vus2i_s: Enum
    r4000allegrex_vus2i_p: Enum
    r4000allegrex_vs2i_s: Enum
    r4000allegrex_vs2i_p: Enum
    r4000allegrex_vi2uc_q: Enum
    r4000allegrex_vi2c_q: Enum
    r4000allegrex_vi2us_p: Enum
    r4000allegrex_vi2us_q: Enum
    r4000allegrex_vi2s_p: Enum
    r4000allegrex_vi2s_q: Enum
    r4000allegrex_vsrt1_q: Enum
    r4000allegrex_vsrt2_q: Enum
    r4000allegrex_vbfy1_p: Enum
    r4000allegrex_vbfy1_q: Enum
    r4000allegrex_vbfy2_q: Enum
    r4000allegrex_vocp_s: Enum
    r4000allegrex_vocp_p: Enum
    r4000allegrex_vocp_t: Enum
    r4000allegrex_vocp_q: Enum
    r4000allegrex_vsocp_s: Enum
    r4000allegrex_vsocp_p: Enum
    r4000allegrex_vfad_p: Enum
    r4000allegrex_vfad_t: Enum
    r4000allegrex_vfad_q: Enum
    r4000allegrex_vavg_p: Enum
    r4000allegrex_vavg_t: Enum
    r4000allegrex_vavg_q: Enum
    r4000allegrex_vsrt3_q: Enum
    r4000allegrex_vsrt4_q: Enum
    r4000allegrex_vsgn_s: Enum
    r4000allegrex_vsgn_p: Enum
    r4000allegrex_vsgn_t: Enum
    r4000allegrex_vsgn_q: Enum
    r4000allegrex_vmfvc: Enum
    r4000allegrex_vmtvc: Enum
    r4000allegrex_vt4444_q: Enum
    r4000allegrex_vt5551_q: Enum
    r4000allegrex_vt5650_q: Enum
    r4000allegrex_vcst_s: Enum
    r4000allegrex_vcst_p: Enum
    r4000allegrex_vcst_t: Enum
    r4000allegrex_vcst_q: Enum
    r4000allegrex_vf2in_s: Enum
    r4000allegrex_vf2in_p: Enum
    r4000allegrex_vf2in_t: Enum
    r4000allegrex_vf2in_q: Enum
    r4000allegrex_vf2iz_s: Enum
    r4000allegrex_vf2iz_p: Enum
    r4000allegrex_vf2iz_t: Enum
    r4000allegrex_vf2iz_q: Enum
    r4000allegrex_vf2iu_s: Enum
    r4000allegrex_vf2iu_p: Enum
    r4000allegrex_vf2iu_t: Enum
    r4000allegrex_vf2iu_q: Enum
    r4000allegrex_vf2id_s: Enum
    r4000allegrex_vf2id_p: Enum
    r4000allegrex_vf2id_t: Enum
    r4000allegrex_vf2id_q: Enum
    r4000allegrex_vi2f_s: Enum
    r4000allegrex_vi2f_p: Enum
    r4000allegrex_vi2f_t: Enum
    r4000allegrex_vi2f_q: Enum
    r4000allegrex_vcmovt_s: Enum
    r4000allegrex_vcmovt_p: Enum
    r4000allegrex_vcmovt_t: Enum
    r4000allegrex_vcmovt_q: Enum
    r4000allegrex_vcmovf_s: Enum
    r4000allegrex_vcmovf_p: Enum
    r4000allegrex_vcmovf_t: Enum
    r4000allegrex_vcmovf_q: Enum
    r4000allegrex_vpfxs: Enum
    r4000allegrex_vpfxt: Enum
    r4000allegrex_vpfxd: Enum
    r4000allegrex_viim_s: Enum
    r4000allegrex_vfim_s: Enum
    r4000allegrex_vmmul_p: Enum
    r4000allegrex_vmmul_t: Enum
    r4000allegrex_vmmul_q: Enum
    r4000allegrex_vhtfm2_p: Enum
    r4000allegrex_vtfm2_p: Enum
    r4000allegrex_vhtfm3_t: Enum
    r4000allegrex_vtfm3_t: Enum
    r4000allegrex_vhtfm4_q: Enum
    r4000allegrex_vtfm4_q: Enum
    r4000allegrex_vmscl_p: Enum
    r4000allegrex_vmscl_t: Enum
    r4000allegrex_vmscl_q: Enum
    r4000allegrex_vcrsp_t: Enum
    r4000allegrex_vqmul_q: Enum
    r4000allegrex_vrot_p: Enum
    r4000allegrex_vrot_t: Enum
    r4000allegrex_vrot_q: Enum
    r4000allegrex_vmmov_p: Enum
    r4000allegrex_vmmov_t: Enum
    r4000allegrex_vmmov_q: Enum
    r4000allegrex_vmidt_p: Enum
    r4000allegrex_vmidt_t: Enum
    r4000allegrex_vmidt_q: Enum
    r4000allegrex_vmzero_p: Enum
    r4000allegrex_vmzero_t: Enum
    r4000allegrex_vmzero_q: Enum
    r4000allegrex_vmone_p: Enum
    r4000allegrex_vmone_t: Enum
    r4000allegrex_vmone_q: Enum
    r4000allegrex_vnop: Enum
    r4000allegrex_vsync: Enum
    r4000allegrex_vflush: Enum
    r4000allegrex_svl_q: Enum
    r4000allegrex_svr_q: Enum
    r4000allegrex_MAX: Enum
    r5900_INVALID: Enum
    r5900_lq: Enum
    r5900_sq: Enum
    r5900_lqc2: Enum
    r5900_sqc2: Enum
    r5900_sync_p: Enum
    r5900_mult: Enum
    r5900_mfsa: Enum
    r5900_mtsa: Enum
    r5900_mtsab: Enum
    r5900_mtsah: Enum
    r5900_madd: Enum
    r5900_maddu: Enum
    r5900_plzcw: Enum
    r5900_mfhi1: Enum
    r5900_mthi1: Enum
    r5900_mflo1: Enum
    r5900_mtlo1: Enum
    r5900_mult1: Enum
    r5900_multu1: Enum
    r5900_div1: Enum
    r5900_divu1: Enum
    r5900_madd1: Enum
    r5900_maddu1: Enum
    r5900_psllh: Enum
    r5900_psrlh: Enum
    r5900_psrah: Enum
    r5900_psllw: Enum
    r5900_psrlw: Enum
    r5900_psraw: Enum
    r5900_paddw: Enum
    r5900_psubw: Enum
    r5900_pcgtw: Enum
    r5900_pmaxw: Enum
    r5900_paddh: Enum
    r5900_psubh: Enum
    r5900_pcgth: Enum
    r5900_pmaxh: Enum
    r5900_paddb: Enum
    r5900_psubb: Enum
    r5900_pcgtb: Enum
    r5900_paddsw: Enum
    r5900_psubsw: Enum
    r5900_pextlw: Enum
    r5900_ppacw: Enum
    r5900_paddsh: Enum
    r5900_psubsh: Enum
    r5900_pextlh: Enum
    r5900_ppach: Enum
    r5900_paddsb: Enum
    r5900_psubsb: Enum
    r5900_pextlb: Enum
    r5900_ppacb: Enum
    r5900_pext5: Enum
    r5900_ppac5: Enum
    r5900_pabsw: Enum
    r5900_pceqw: Enum
    r5900_pminw: Enum
    r5900_padsbh: Enum
    r5900_pabsh: Enum
    r5900_pceqh: Enum
    r5900_pminh: Enum
    r5900_pceqb: Enum
    r5900_padduw: Enum
    r5900_psubuw: Enum
    r5900_pextuw: Enum
    r5900_padduh: Enum
    r5900_psubuh: Enum
    r5900_pextuh: Enum
    r5900_paddub: Enum
    r5900_psubub: Enum
    r5900_pextub: Enum
    r5900_qfsrv: Enum
    r5900_pmaddw: Enum
    r5900_psllvw: Enum
    r5900_psrlvw: Enum
    r5900_pmsubw: Enum
    r5900_pmfhi: Enum
    r5900_pmflo: Enum
    r5900_pinth: Enum
    r5900_pmultw: Enum
    r5900_pdivw: Enum
    r5900_pcpyld: Enum
    r5900_pmaddh: Enum
    r5900_phmadh: Enum
    r5900_pand: Enum
    r5900_pxor: Enum
    r5900_pmsubh: Enum
    r5900_phmsbh: Enum
    r5900_pexeh: Enum
    r5900_prevh: Enum
    r5900_pmulth: Enum
    r5900_pdivbw: Enum
    r5900_pexew: Enum
    r5900_prot3w: Enum
    r5900_pmadduw: Enum
    r5900_psravw: Enum
    r5900_pmthi: Enum
    r5900_pmtlo: Enum
    r5900_pinteh: Enum
    r5900_pmultuw: Enum
    r5900_pdivuw: Enum
    r5900_pcpyud: Enum
    r5900_por: Enum
    r5900_pnor: Enum
    r5900_pexch: Enum
    r5900_pcpyh: Enum
    r5900_pexcw: Enum
    r5900_pmfhl_lw: Enum
    r5900_pmfhl_uw: Enum
    r5900_pmfhl_slw: Enum
    r5900_pmfhl_lh: Enum
    r5900_pmfhl_sh: Enum
    r5900_pmthl_lw: Enum
    r5900_ei: Enum
    r5900_di: Enum
    r5900_c1__sqrt_s: Enum
    r5900_rsqrt_s: Enum
    r5900_adda_s: Enum
    r5900_suba_s: Enum
    r5900_mula_s: Enum
    r5900_madd_s: Enum
    r5900_msub_s: Enum
    r5900_madda_s: Enum
    r5900_msuba_s: Enum
    r5900_max_s: Enum
    r5900_min_s: Enum
    r5900_c_lt_s: Enum
    r5900_c_le_s: Enum
    r5900_qmfc2_ni: Enum
    r5900_cfc2_ni: Enum
    r5900_qmtc2_ni: Enum
    r5900_ctc2_ni: Enum
    r5900_qmfc2_i: Enum
    r5900_cfc2_i: Enum
    r5900_qmtc2_i: Enum
    r5900_ctc2_i: Enum
    r5900_bc2f: Enum
    r5900_bc2t: Enum
    r5900_bc2fl: Enum
    r5900_bc2tl: Enum
    r5900_vaddx: Enum
    r5900_vaddy: Enum
    r5900_vaddz: Enum
    r5900_vaddw: Enum
    r5900_vsubx: Enum
    r5900_vsuby: Enum
    r5900_vsubz: Enum
    r5900_vsubw: Enum
    r5900_vmaddx: Enum
    r5900_vmaddy: Enum
    r5900_vmaddz: Enum
    r5900_vmaddw: Enum
    r5900_vmsubx: Enum
    r5900_vmsuby: Enum
    r5900_vmsubz: Enum
    r5900_vmsubw: Enum
    r5900_vmaxx: Enum
    r5900_vmaxy: Enum
    r5900_vmaxz: Enum
    r5900_vmaxw: Enum
    r5900_vminix: Enum
    r5900_vminiy: Enum
    r5900_vminiz: Enum
    r5900_vminiw: Enum
    r5900_vmulx: Enum
    r5900_vmuly: Enum
    r5900_vmulz: Enum
    r5900_vmulw: Enum
    r5900_vmulq: Enum
    r5900_vmaxi: Enum
    r5900_vmuli: Enum
    r5900_vminii: Enum
    r5900_vaddq: Enum
    r5900_vmaddq: Enum
    r5900_vaddi: Enum
    r5900_vmaddi: Enum
    r5900_vsubq: Enum
    r5900_vmsubq: Enum
    r5900_vsubi: Enum
    r5900_vmsubi: Enum
    r5900_vadd: Enum
    r5900_vmadd: Enum
    r5900_vmul: Enum
    r5900_vmax: Enum
    r5900_vsub: Enum
    r5900_vmsub: Enum
    r5900_vopmsub: Enum
    r5900_vmini: Enum
    r5900_viadd: Enum
    r5900_visub: Enum
    r5900_viaddi: Enum
    r5900_viand: Enum
    r5900_vior: Enum
    r5900_vcallms: Enum
    r5900_vcallmsr: Enum
    r5900_vaddax: Enum
    r5900_vadday: Enum
    r5900_vaddaz: Enum
    r5900_vaddaw: Enum
    r5900_vsubax: Enum
    r5900_vsubay: Enum
    r5900_vsubaz: Enum
    r5900_vsubaw: Enum
    r5900_vmaddax: Enum
    r5900_vmadday: Enum
    r5900_vmaddaz: Enum
    r5900_vmaddaw: Enum
    r5900_vmsubax: Enum
    r5900_vmsubay: Enum
    r5900_vmsubaz: Enum
    r5900_vmsubaw: Enum
    r5900_vitof0: Enum
    r5900_vitof4: Enum
    r5900_vitof12: Enum
    r5900_vitof15: Enum
    r5900_vftoi0: Enum
    r5900_vftoi4: Enum
    r5900_vftoi12: Enum
    r5900_vftoi15: Enum
    r5900_vmulax: Enum
    r5900_vmulay: Enum
    r5900_vmulaz: Enum
    r5900_vmulaw: Enum
    r5900_vmulaq: Enum
    r5900_vabs: Enum
    r5900_vmulai: Enum
    r5900_vclipw: Enum
    r5900_vaddaq: Enum
    r5900_vmaddaq: Enum
    r5900_vaddai: Enum
    r5900_vmaddai: Enum
    r5900_vsubaq: Enum
    r5900_vmsubaq: Enum
    r5900_vsubai: Enum
    r5900_vmsubai: Enum
    r5900_vadda: Enum
    r5900_vmadda: Enum
    r5900_vmula: Enum
    r5900_vsuba: Enum
    r5900_vmsuba: Enum
    r5900_vopmula: Enum
    r5900_vnop: Enum
    r5900_vmove: Enum
    r5900_vmr32: Enum
    r5900_vlqi: Enum
    r5900_vsqi: Enum
    r5900_vlqd: Enum
    r5900_vsqd: Enum
    r5900_vdiv: Enum
    r5900_vsqrt: Enum
    r5900_vrsqrt: Enum
    r5900_vwaitq: Enum
    r5900_vmtir: Enum
    r5900_vmfir: Enum
    r5900_vrnext: Enum
    r5900_vrget: Enum
    r5900_vrinit: Enum
    r5900_vrxor: Enum
    r5900_vilwr_w: Enum
    r5900_vilwr_z: Enum
    r5900_vilwr_y: Enum
    r5900_vilwr_x: Enum
    r5900_viswr_w: Enum
    r5900_viswr_z: Enum
    r5900_viswr_y: Enum
    r5900_viswr_x: Enum
    r5900_MAX: Enum
    ALL_MAX: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .Enum import Enum
from .Abi import Abi

class _RabbitizerConfig:
    regNames_namedRegisters: bool = True
    regNames_gprAbiNames: Enum = Abi.O32
    regNames_fprAbiNames: Enum = Abi.NUMERIC
    regNames_userFpcCsr: bool = True
    regNames_vr4300Cop0NamedRegisters: bool = True
    regNames_vr4300RspCop0NamedRegisters: bool = True
    regNames_r4000AllegrexVfpuControlNamedRegisters: bool = False

    pseudos_enablePseudos: bool = True
    pseudos_pseudoBeqz: bool = True
    pseudos_pseudoBnez: bool = True
    pseudos_pseudoB: bool = True
    pseudos_pseudoMove: bool = True
    pseudos_pseudoNot: bool = True
    pseudos_pseudoNeg: bool = True
    pseudos_pseudoNegu: bool = True
    pseudos_pseudoBal: bool = True

    toolchainTweaks_treatJAsUnconditionalBranch: bool = True
    toolchainTweaks_sn64DivFix: bool = False
    toolchainTweaks_gnuMode: bool = True
    toolchainTweaks_r5900ProdgSnAsInvertedRegs: bool = False

    misc_opcodeLJust: int = 11
    misc_unknownInstrComment: bool = True
    misc_omit0XOnSmallImm: bool = False
    misc_upperCaseImm: bool = True
    misc_expandJalr: bool = False

config: _RabbitizerConfig



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class OperandType:
    ALL_INVALID: Enum
    cpu_rs: Enum
    cpu_rt: Enum
    cpu_rd: Enum
    cpu_sa: Enum
    cpu_zero: Enum
    cpu_cop0d: Enum
    cpu_fs: Enum
    cpu_ft: Enum
    cpu_fd: Enum
    cpu_cop1cs: Enum
    cpu_cop2t: Enum
    cpu_cop2cd: Enum
    cpu_op: Enum
    cpu_hint: Enum
    cpu_code: Enum
    cpu_code_lower: Enum
    cpu_copraw: Enum
    cpu_label: Enum
    cpu_immediate: Enum
    cpu_branch_target_label: Enum
    cpu_immediate_base: Enum
    cpu_maybe_rd_rs: Enum
    rsp_rs: Enum
    rsp_rt: Enum
    rsp_rd: Enum
    rsp_cop0d: Enum
    rsp_cop2t: Enum
    rsp_cop2cd: Enum
    rsp_vs: Enum
    rsp_vt: Enum
    rsp_vd: Enum
    rsp_hint: Enum
    rsp_vt_elementhigh: Enum
    rsp_vt_elementlow: Enum
    rsp_vd_de: Enum
    rsp_vs_index: Enum
    rsp_offset_rs: Enum
    rsp_immediate_base: Enum
    rsp_maybe_rd_rs: Enum
    r3000gte_gbg: Enum
    r3000gte_sf: Enum
    r3000gte_mx: Enum
    r3000gte_v: Enum
    r3000gte_cv: Enum
    r3000gte_lm: Enum
    r4000allegrex_s_vs: Enum
    r4000allegrex_s_vt: Enum
    r4000allegrex_s_vd: Enum
    r4000allegrex_s_vt_imm: Enum
    r4000allegrex_s_vd_imm: Enum
    r4000allegrex_p_vs: Enum
    r4000allegrex_p_vt: Enum
    r4000allegrex_p_vd: Enum
    r4000allegrex_t_vs: Enum
    r4000allegrex_t_vt: Enum
    r4000allegrex_t_vd: Enum
    r4000allegrex_q_vs: Enum
    r4000allegrex_q_vt: Enum
    r4000allegrex_q_vd: Enum
    r4000allegrex_q_vt_imm: Enum
    r4000allegrex_mp_vs: Enum
    r4000allegrex_mp_vt: Enum
    r4000allegrex_mp_vd: Enum
    r4000allegrex_mp_vs_transpose: Enum
    r4000allegrex_mt_vs: Enum
    r4000allegrex_mt_vt: Enum
    r4000allegrex_mt_vd: Enum
    r4000allegrex_mt_vs_transpose: Enum
    r4000allegrex_mq_vs: Enum
    r4000allegrex_mq_vt: Enum
    r4000allegrex_mq_vd: Enum
    r4000allegrex_mq_vs_transpose: Enum
    r4000allegrex_cop2cs: Enum
    r4000allegrex_cop2cd: Enum
    r4000allegrex_pos: Enum
    r4000allegrex_size: Enum
    r4000allegrex_size_plus_pos: Enum
    r4000allegrex_imm3: Enum
    r4000allegrex_offset14_base: Enum
    r4000allegrex_offset14_base_maybe_wb: Enum
    r4000allegrex_vcmp_cond: Enum
    r4000allegrex_vcmp_cond_s_maybe_vs_maybe_vt: Enum
    r4000allegrex_vcmp_cond_p_maybe_vs_maybe_vt: Enum
    r4000allegrex_vcmp_cond_t_maybe_vs_maybe_vt: Enum
    r4000allegrex_vcmp_cond_q_maybe_vs_maybe_vt: Enum
    r4000allegrex_vconstant: Enum
    r4000allegrex_power_of_two: Enum
    r4000allegrex_vfpu_cc_bit: Enum
    r4000allegrex_bn: Enum
    r4000allegrex_int16: Enum
    r4000allegrex_float16: Enum
    r4000allegrex_p_vrot_code: Enum
    r4000allegrex_t_vrot_code: Enum
    r4000allegrex_q_vrot_code: Enum
    r4000allegrex_rpx: Enum
    r4000allegrex_rpy: Enum
    r4000allegrex_rpz: Enum
    r4000allegrex_rpw: Enum
    r4000allegrex_wpx: Enum
    r4000allegrex_wpy: Enum
    r4000allegrex_wpz: Enum
    r4000allegrex_wpw: Enum
    r5900_I: Enum
    r5900_Q: Enum
    r5900_R: Enum
    r5900_ACC: Enum
    r5900_ACCxyzw: Enum
    r5900_vfs: Enum
    r5900_vft: Enum
    r5900_vfd: Enum
    r5900_vfsxyzw: Enum
    r5900_vftxyzw: Enum
    r5900_vfdxyzw: Enum
    r5900_vfsxyzw_inv_vft: Enum
    r5900_vftxyzw_inv_vfs: Enum
    r5900_vfsn: Enum
    r5900_vftn: Enum
    r5900_vfdn: Enum
    r5900_vfsl: Enum
    r5900_vftl: Enum
    r5900_vfdl: Enum
    r5900_vfsm: Enum
    r5900_vftm: Enum
    r5900_vfdm: Enum
    r5900_vis: Enum
    r5900_vit: Enum
    r5900_vid: Enum
    r5900_vis_predecr: Enum
    r5900_vit_predecr: Enum
    r5900_vid_predecr: Enum
    r5900_vis_postincr: Enum
    r5900_vit_postincr: Enum
    r5900_vid_postincr: Enum
    r5900_vis_parenthesis: Enum
    r5900_immediate5: Enum
    r5900_immediate15: Enum
    ALL_MAX: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .Enum import Enum
from .InstrCategory import InstrCategory
from .TrinaryValue import TrinaryValue


class Instruction:
    rs: Enum
    """The value of the `rs` register for this instruction.
    The type of this attribute will be either a `RegGprO32` or a `RegGprN32` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `rs` register, then a Runtime exception will be raised.
    Read-only."""
    rt: Enum
    """The value of the `rt` register for this instruction.
    The type of this attribute will be either a `RegGprO32` or a `RegGprN32` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `rt` register, then a Runtime exception will be raised.
    Read-only."""
    rd: Enum
    """The value of the `rd` register for this instruction.
    The type of this attribute will be either a `RegGprO32` or a `RegGprN32` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `rd` register, then a Runtime exception will be raised.
    Read-only."""
    sa: int
    """The value of the `sa` field for this instruction.
    If the current instruction does not have a `sa` field, then a Runtime exception will be raised.
    Read-only."""
    fs: Enum
    """The value of the `fs` register for this instruction.
    The type of this attribute will be either a `RegGprO32`, `RegGprN32` or a `RegGprN64` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `fs` register, then a Runtime exception will be raised.
    Read-only."""
    ft: Enum
    """The value of the `ft` register for this instruction.
    The type of this attribute will be either a `RegGprO32`, `RegGprN32` or a `RegGprN64` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `ft` register, then a Runtime exception will be raised.
    Read-only."""
    fd: Enum
    """The value of the `fd` register for this instruction.
    The type of this attribute will be either a `RegGprO32`, `RegGprN32` or a `RegGprN64` depending on the current `config.regNames_gprAbiNames` value.
    If the current instruction does not use the `fd` register, then a Runtime exception will be raised.
    Read-only."""

    uniqueId: Enum
    """An unique identificator for the opcode of this instruction.
    The type is an `InstrId` enum.
    Read-only."""

    instrIdType: Enum
    """An identificator for the general type for the opcode of this instruction.
    The type is an `InstrIdType` enum.
    Read-only."""

    vram: int = 0
    """The vram (virtual ram) address for this instruction"""
    inHandwrittenFunction: bool = False
    """Boolean value indicating if the current instruction is used on a handwritten function. This is intended to be determined by the user."""

    flag_r5900DisasmAsData: Enum = TrinaryValue.NONE
    """Flag to override the r5900DisasmAsData global configuration.

    - This flag allows to fine-tune R5900 instruction set that are affected by the global `gnuMode` option.
        - Currently these instructions are: `trunc.w.s` (r5900 mode), `cvt.w.s` (r5900 mode), `vclipw` and `vsqrt`.

    - `TrinaryValue.TRUE` forces the instruction to be disassembled as data.
    - `TrinaryValue.FALSE` bypasses the global checks for disassembling a word as data. A word will still be disassembled as data if it can't be decoded.
    - `TrinaryValue.NONE` leaves this decision to the global settings.
    """
    flag_r5900UseDollar: Enum = TrinaryValue.NONE
    """Flag to override the gnuMode global configuration for specific R5900 VU instructions.

    - `TrinaryValue.TRUE` forces the use of dollar signs ($) on R5900's VU instructions.
    - `TrinaryValue.FALSE` forces disassembling to not use of dollar signs ($) on R5900's VU instructions.
    - `TrinaryValue.NONE` leaves this decision to the global settings.
    """
    flag_r5900ProdgSnAsInvertedRegs: Enum = TrinaryValue.NONE
    """Flag to override the  global configuration.

    - `TrinaryValue.TRUE` forces the register order used by ProDG SN AS for `vadda` and `vmsuba`.
    - `TrinaryValue.FALSE` forces the standard register order for those instructions.
    - `TrinaryValue.NONE` leaves this decision to the global settings.
    """


    def __init__(self, word: int, vram: int=0, category: Enum=InstrCategory.CPU) -> None:
        """Decode an Instruction encoded as the 4 bytes `word`, located at `vram`.

        `vram` is used to decode jump instructions that use the PC value to get the upper bits of the target address.
        """

    def getRaw(self) -> int:
        """Get the word value encoding the instruction.

        The returned value may not be the same as the one to instance this Instruction
        if a method that modifies the word has been used, like `instr.blankOut()`.
        """
    def getImmediate(self) -> int: #! deprecated
        """Use `getProcessedImmediate()` instead"""
    def getProcessedImmediate(self) -> int:
        """Get the (possibly signed) immediate value for this instruction.

        This only makes sense for an instruction with an immediate,
        which can be checked with `instr.hasOperandAlias(OperandType.cpu_immediate)`.

        An exception will be raised if the instruction does not contain an immediate field.
        """
    def getInstrIndexAsVram(self) -> int:
        """Get the target vram address this instruction jumps to.
        This method is intended only for direct jump instructions.

        This only makes sense if the instruction is a direct jump,
        which can be checked with `instr.isJumpWithAddress()`.

        An exception will be raised if the instruction is not a jump instruction.
        """
    def getBranchOffset(self) -> int:
        """Returns the offset (in bytes) that the branch instruction would branch,
        relative to the instruction itself. This method is intended only for branch
        instructions.

        The returned value can be negative, meaning the branch instructions does
        a backwards branch.

        This only makes sense for an instruction is a branch,
        which can be checked with `instr.isBranch()`.

        To get the branch offset of either a branch instruction or a jump instruction
        use `instr.getBranchOffsetGeneric()` instead.

        An exception will be raised if the instruction is not a branch instruction.
        """
    def getGenericBranchOffset(self, currentVram: int) -> int: #! deprecated
        """Use `getBranchOffsetGeneric()` instead"""
    def getBranchOffsetGeneric(self) -> int:
        """Returns the offset (in bytes) that the instruction would branch,
        relative to the instruction itself. This method is intended for both branch
        and jump instructions.

        The returned value can be either positive or negative.

        This only makes sense for an instruction is a branch or a direct jump,
        which can be checked with `instr.isBranch()` or `instr.isJumpWithAddress()`.

        An exception will be raised if the instruction is neither a branch or a
        jump instruction.
        """
    def getBranchVramGeneric(self) -> int:
        """Get the target vram address this instruction jumps to.
        This method is intended only for branch or direct jump instructions.

        This only makes sense for an instruction is a branch or a direct jump,
        which can be checked with `instr.isBranch()` or `instr.isJumpWithAddress()`.

        An exception will be raised if the instruction is neither a branch or a
        jump instruction.
        """
    def getDestinationGpr(self) -> Enum|None:
        """
        Returns the general purpose register (GPR) which this instruction modifies,
        or `None` if the instruction does not modify the state of any GPR
        """
    def outputsToGprZero(self) -> bool:
        """
        Returns `True` if the GPR which is modified by this register is $zero,
        `False` otherwise.
        Returns `false` if this instruction does not modify a GPR.
        """
    def getOpcodeName(self) -> str:
        """Returns the mnemonic of the instruction.
        """

    def blankOut(self) -> None:
        """Zero'es out every field (registers and immediate) of the instruction
        leaving only the mnemonic.
        """

    def isImplemented(self) -> bool: #! deprecated
        """Use `instr.isValid()` instead"""
    def isLikelyHandwritten(self) -> bool:
        """Try to guess if the given instruction was handwritten by a human
        instead of generated by a C compiler.
        """
    def isNop(self) -> bool:
        """Check if the instruction is literally the `nop` instruction."""
    def isUnconditionalBranch(self) -> bool:
        """Check if the instruction is an instruction that will always branch unconditionally.

        This is always true for the `b` instruction.

        Some compilers use the `j` instruction for unconditional branches instead
        of the `b` instruction. Treating this instruction as an unconditional branch
        can be configured with the `config.toolchainTweaks_treatJAsUnconditionalBranch`
        option.
        """
    def isFunctionCall(self) -> bool:
        """Check if this is an instruction used for function calls.

        This is always true for "and link" instructions.

        Some compilers use the `j` instruction for tail call optimizations, meaning
        we may require to give special treatment to this instruction if we are
        analyzing code emitted by one of those compilers, like clearing registers
        after a tail call. This can be configured by turning off the
        `config.toolchainTweaks_treatJAsUnconditionalBranch` option.
        """

    def isReturn(self) -> bool:
        """Check if the instruction and its register is the one usually used for
        returning from a function.

        Specfically, this checks if the instruction is a `jr $ra`.

        Returns `False` if the instruction is not a `jr` or if it is a `jr` but
        the register is not `$ra`.
        """
    def isJumptableJump(self) -> bool:
        """Check if the instruction and its register is the one usually used for
        jumping with jumptables.

        Specfically, this checks if the instruction is a `jr` but its register is
        not `$ra`.

        Returns `False` if the instruction is not a `jr` or if it is a `jr` but
        the register is `$ra`.
        """

    def isJrRa(self) -> bool: #! deprecated
        """Use `isReturn()` instead"""
    def isJrNotRa(self) -> bool: #! deprecated
        """Use `isJumptableJump()` instead"""

    def hasDelaySlot(self) -> bool:
        """Check if the instruction has a delay slot."""
    def mapInstrToType(self) -> str|None: #! deprecated
        """use `getAccessType()` instead"""

    def sameOpcode(self, other: Instruction) -> bool:
        """Check if two instructions have the same mnemonic"""
    def sameOpcodeButDifferentArguments(self, other: Instruction) -> bool:
        """Check if two instructions have the same mnemonic but their arguments
        are different.
        """

    def hasOperand(self, operand: Enum) -> bool:
        """Check if the instruction has specifically the `operand`.

        `operand` should be from the `OperandType` enum.
        """
    def hasOperandAlias(self, operand: Enum) -> bool:
        """Check if the instruction has the `operand` or an alias of it.

        (if unsure whether to use `hasOperand` or `hasOperandAlias`, use `hasOperandAlias`)

        `operand` should be from the `OperandType` enum.
        """

    def isValid(self) -> bool:
        """Check the word corresponds to a valid instruction."""

    #! deprecated
    def isUnknownType(self) -> bool: ...
    #! deprecated
    def isJType(self) -> bool:
        """Use `isJumpWithAddress()` instead"""
    #! deprecated
    def isIType(self) -> bool:
        """Use `hasOperandAlias(OperandType.cpu_immediate)` instead"""
    #! deprecated
    def isRType(self) -> bool: ...
    #! deprecated
    def isRegimmType(self) -> bool:
        """Use `hasOperandAlias(OperandType.cpu_immediate)` instead"""

    def isBranch(self) -> bool:
        """Instruction is a branch instruction.

        This is also true for branch likely instructions.
        """
    def isBranchLikely(self) -> bool: ...
    def isJump(self) -> bool: ...
    def isJumpWithAddress(self) -> bool: ...
    def isTrap(self) -> bool: ...
    def isFloat(self) -> bool: ...
    def isDouble(self) -> bool: ...
    def isUnsigned(self) -> bool: ...
    def modifiesRs(self) -> bool: ...
    def modifiesRt(self) -> bool: ...
    def modifiesRd(self) -> bool: ...
    def readsRs(self) -> bool: ...
    def readsRt(self) -> bool: ...
    def readsRd(self) -> bool: ...
    def readsHI(self) -> bool: ...
    def readsLO(self) -> bool: ...
    def modifiesHI(self) -> bool: ...
    def modifiesLO(self) -> bool: ...
    def modifiesFs(self) -> bool: ...
    def modifiesFt(self) -> bool: ...
    def modifiesFd(self) -> bool: ...
    def readsFs(self) -> bool: ...
    def readsFt(self) -> bool: ...
    def readsFd(self) -> bool: ...
    def notEmitedByCompilers(self) -> bool: #! deprecated
        """Use `notEmittedByCompilers()` instead"""
    def notEmittedByCompilers(self) -> bool: ...
    def canBeHi(self) -> bool: ...
    def canBeLo(self) -> bool: ...
    def doesLink(self) -> bool: ...
    def doesDereference(self) -> bool: ...
    def doesLoad(self) -> bool: ...
    def doesStore(self) -> bool: ...
    def maybeIsMove(self) -> bool: ...
    def isPseudo(self) -> bool: ...
    def getAccessType(self) -> Enum: ...
    def doesUnsignedMemoryAccess(self) -> bool: ...

    def disassemble(self, immOverride: str|None=None, extraLJust: int=0) -> str:
        """Returns a string that can be assembled back to the instruction raw word.
        `immOverride` can be a string that replaces the immediate in the disassembly,
        or the jump target, if any. If the instruction has neither, it is ignored.
        Examples:
        >>> Instruction(0x3C0A0042).disassemble()
        'lui         $t2, 0x42'
        >>> Instruction(0x3C0A0042).disassemble("hex_answer")
        'lui         $t2, hex_answer'
        >>> Instruction(0x0C000042).disassemble()
        'jal         func_80000108'
        >>> Instruction(0x0C000042).disassemble("my_target")
        'jal         my_target'
        """

    def __reduce__(self) -> tuple: ...

    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class Enum:
    name: str
    value: int

    def __init__(self, enumType: str, name: str, value: int) -> None: ...

    def __reduce__(self) -> tuple: ...

    def __lt__(self, __o: object) -> bool: ...
    def __le__(self, __o: object) -> bool: ...
    def __eq__(self, __o: object) -> bool: ...
    def __gt__(self, __o: object) -> bool: ...
    def __ge__(self, __o: object) -> bool: ...

    def __hash__(self) -> int: ...

    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class LoPairingInfo:
    instrOffset: int
    value: int
    shouldProcess: bool
    isGpRel: bool
    isGpGot: bool

    def __init__(self): ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .rabbitizer import Instruction
from .LoPairingInfo import LoPairingInfo
from .JrRegData import JrRegData


class RegistersTracker:
    def __init__(self, other: RegistersTracker|None=None): ...

    def moveRegisters(self, instr: Instruction) -> bool: ...
    def overwriteRegisters(self, instr: Instruction, instructionOffset: int) -> None: ...
    def unsetRegistersAfterFuncCall(self, instr: Instruction, prevInstr: Instruction) -> None: ...
    def getAddressIfCanSetType(self, instr: Instruction, instrOffset: int) -> int|None: ...
    #! deprecated: use `getJrRegData` instead
    def getJrInfo(self, instr: Instruction) -> tuple[int, int]|None: ...
    def getJrRegData(self, instr: Instruction) -> JrRegData: ...

    def processLui(self, instr: Instruction, instrOffset: int, prevInstr: Instruction|None=None) -> None: ...
    def processGpLoad(self, instr: Instruction, instrOffset: int) -> None: ...
    def getLuiOffsetForConstant(self, instr: Instruction) -> int|None: ...
    def processConstant(self, instr: Instruction, value: int, offset: int) -> None: ...
    def getLuiOffsetForLo(self, instr: Instruction, instrOffset: int) -> tuple[int, bool, bool]: #! deprecated
        """Use `.preprocessLoAndGetInfo()` instead"""
    def preprocessLoAndGetInfo(self, instr: Instruction, instrOffset: int) -> LoPairingInfo: ...
    def processLo(self, instr: Instruction, value: int, offset: int) -> None: ...
    def processBranch(self, instr: Instruction, instrOffset: int) -> None: ...
    def hasLoButNoHi(self, instr: Instruction) -> bool: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class JrRegData:
    def hasInfo(self) -> bool: ...
    def offset(self) -> int: ...
    def address(self) -> int: ...
    def checkedForBranching(self) -> bool: ...
    def lastBranchOffset(self) -> bool: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class InstrIdType:
    ALL_INVALID: Enum
    CPU_INVALID: Enum
    CPU_NORMAL: Enum
    CPU_SPECIAL: Enum
    CPU_REGIMM: Enum
    CPU_COP0: Enum
    CPU_COP0_BC0: Enum
    CPU_COP0_TLB: Enum
    CPU_COP1: Enum
    CPU_COP1_BC1: Enum
    CPU_COP1_FPUS: Enum
    CPU_COP1_FPUD: Enum
    CPU_COP1_FPUW: Enum
    CPU_COP1_FPUL: Enum
    CPU_COP2: Enum
    RSP_INVALID: Enum
    RSP_NORMAL: Enum
    RSP_NORMAL_LWC2: Enum
    RSP_NORMAL_SWC2: Enum
    RSP_SPECIAL: Enum
    RSP_REGIMM: Enum
    RSP_COP0: Enum
    RSP_COP1: Enum
    RSP_COP2: Enum
    RSP_COP2_VU: Enum
    R3000GTE_INVALID: Enum
    R3000GTE_NORMAL: Enum
    R3000GTE_SPECIAL: Enum
    R3000GTE_REGIMM: Enum
    R3000GTE_COP0: Enum
    R3000GTE_COP1: Enum
    R3000GTE_COP2: Enum
    R3000GTE_COP2_GTE: Enum
    R4000ALLEGREX_INVALID: Enum
    R4000ALLEGREX_NORMAL: Enum
    R4000ALLEGREX_SPECIAL: Enum
    R4000ALLEGREX_SPECIAL_RS: Enum
    R4000ALLEGREX_SPECIAL_SA: Enum
    R4000ALLEGREX_REGIMM: Enum
    R4000ALLEGREX_SPECIAL2: Enum
    R4000ALLEGREX_SPECIAL3: Enum
    R4000ALLEGREX_SPECIAL3_BSHFL: Enum
    R4000ALLEGREX_COP0: Enum
    R4000ALLEGREX_COP0_BC0: Enum
    R4000ALLEGREX_COP0_TLB: Enum
    R4000ALLEGREX_COP1: Enum
    R4000ALLEGREX_COP1_BC1: Enum
    R4000ALLEGREX_COP1_FPUS: Enum
    R4000ALLEGREX_COP1_FPUW: Enum
    R4000ALLEGREX_COP2: Enum
    R4000ALLEGREX_COP2_BC2: Enum
    R4000ALLEGREX_COP2_MFHC2: Enum
    R4000ALLEGREX_COP2_MFHC2_P: Enum
    R4000ALLEGREX_COP2_MFHC2_P_S: Enum
    R4000ALLEGREX_COP2_MTHC2: Enum
    R4000ALLEGREX_VFPU0: Enum
    R4000ALLEGREX_VFPU1: Enum
    R4000ALLEGREX_VFPU3: Enum
    R4000ALLEGREX_VFPU4: Enum
    R4000ALLEGREX_VFPU4_FMT0: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT0: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT2: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT3: Enum
    R4000ALLEGREX_VFPU4_FMT0_RND: Enum
    R4000ALLEGREX_VFPU4_FMT0_CVTFLT: Enum
    R4000ALLEGREX_VFPU4_FMT0_CVTINT: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT8: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT9: Enum
    R4000ALLEGREX_VFPU4_FMT0_CONTROL: Enum
    R4000ALLEGREX_VFPU4_FMT0_COLOR: Enum
    R4000ALLEGREX_VFPU4_FMT0_CST: Enum
    R4000ALLEGREX_VFPU4_FMT2: Enum
    R4000ALLEGREX_VFPU4_FMT2_CNDMOVE: Enum
    R4000ALLEGREX_VFPU5: Enum
    R4000ALLEGREX_VFPU6: Enum
    R4000ALLEGREX_VFPU6_FMT7: Enum
    R4000ALLEGREX_VFPU6_FMT7_FMT0: Enum
    R4000ALLEGREX_VFPU7: Enum
    R4000ALLEGREX_QUADLR: Enum
    R5900_INVALID: Enum
    R5900_NORMAL: Enum
    R5900_SPECIAL: Enum
    R5900_REGIMM: Enum
    R5900_COP0: Enum
    R5900_COP0_TLB: Enum
    R5900_COP1: Enum
    R5900_COP1_FPUS: Enum
    R5900_COP2: Enum
    R5900_COP2_NOHIGHBIT: Enum
    R5900_COP2_BC2: Enum
    R5900_COP2_SPECIAL1: Enum
    R5900_COP2_SPECIAL2: Enum
    R5900_COP2_VIWR: Enum
    R5900_MMI: Enum
    R5900_MMI_0: Enum
    R5900_MMI_1: Enum
    R5900_MMI_2: Enum
    R5900_MMI_3: Enum
    R5900_MMI_PMFHL: Enum
    R5900_MMI_PMTHL: Enum
    ALL_MAX: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2023-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class TrinaryValue:
    NONE: Enum
    FALSE: Enum
    TRUE: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegCop1O32:
    fv0: Enum
    fv0f: Enum
    fv1: Enum
    fv1f: Enum
    ft0: Enum
    ft0f: Enum
    ft1: Enum
    ft1f: Enum
    ft2: Enum
    ft2f: Enum
    ft3: Enum
    ft3f: Enum
    fa0: Enum
    fa0f: Enum
    fa1: Enum
    fa1f: Enum
    ft4: Enum
    ft4f: Enum
    ft5: Enum
    ft5f: Enum
    fs0: Enum
    fs0f: Enum
    fs1: Enum
    fs1f: Enum
    fs2: Enum
    fs2f: Enum
    fs3: Enum
    fs3f: Enum
    fs4: Enum
    fs4f: Enum
    fs5: Enum
    fs5f: Enum



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class InstrCategory:
    CPU: Enum
    RSP: Enum
    R3000GTE: Enum
    R4000ALLEGREX: Enum
    R5900: Enum
    MAX: Enum
    @staticmethod
    def fromStr(name: str | None) -> Enum|None: ...



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegCop1N32:
    fv0: Enum
    ft14: Enum
    fv1: Enum
    ft15: Enum
    ft0: Enum
    ft1: Enum
    ft2: Enum
    ft3: Enum
    ft4: Enum
    ft5: Enum
    ft6: Enum
    ft7: Enum
    fa0: Enum
    fa1: Enum
    fa2: Enum
    fa3: Enum
    fa4: Enum
    fa5: Enum
    fa6: Enum
    fa7: Enum
    fs0: Enum
    ft8: Enum
    fs1: Enum
    ft9: Enum
    fs2: Enum
    ft10: Enum
    fs3: Enum
    ft11: Enum
    fs4: Enum
    ft12: Enum
    fs5: Enum
    ft13: Enum
